import { SmartForm, createSmartForm, useCustomItem } from '@s-form/core'
import { mount } from '@vue/test-utils'
import { expect, test } from 'vitest'
import CustomModelComp from './CustomModelComp.vue'
import BasicComp from './BasicComp.vue'
import { reactive } from 'vue'

test('CustomModelComp - use modelKeyMap', async () => {
  const model = reactive({ name: '', age: 12 })
  const wrapper = mount(SmartForm, {
    props: {
      formSchema: [
        useCustomItem({
          component: CustomModelComp,
          key: 'name',
          modelKeyMap: {
            model: 'name',
            count: 'age',
          },
        }),
      ],
      modelValue: model,
    },
    global: {
      plugins: [createSmartForm()],
    },
  })

  await wrapper.find('input').setValue('hi')
  expect(model.name).toBe('hi')
  expect(
    wrapper.getComponent(SmartForm).vm.$.exposed!.getResult()
  ).resolves.toStrictEqual({ name: 'hi', age: 12 })
  await wrapper.find('input[type="number"]').setValue('18')
  expect(model.age).toBe(18)
  expect(
    wrapper.getComponent(SmartForm).vm.$.exposed!.getResult()
  ).resolves.toStrictEqual({ name: 'hi', age: 18 })
})

test('CustomModelComp - use modelValueKey', async () => {
  const model = reactive({ name: '' })
  const wrapper = mount(SmartForm, {
    props: {
      formSchema: [
        useCustomItem({
          component: CustomModelComp,
          key: 'name',
          modelValueKey: 'model',
        }),
      ],
      modelValue: model,
    },
    global: {
      plugins: [createSmartForm()],
    },
  })

  await wrapper.find('input').setValue('hi')
  expect(model.name).toBe('hi')
  expect(
    wrapper.getComponent(SmartForm).vm.$.exposed!.getResult()
  ).resolves.toStrictEqual({ name: 'hi' })
})

test('BasicComp', async () => {
  const model = reactive({ name: '' })
  const wrapper = mount(SmartForm, {
    props: {
      formSchema: [
        useCustomItem({
          component: BasicComp,
          key: 'name',
        }),
      ],
      modelValue: model,
    },
    global: {
      plugins: [createSmartForm()],
    },
  })

  await wrapper.find('input').setValue('hi')
  expect(model.name).toBe('hi')
  expect(
    wrapper.getComponent(SmartForm).vm.$.exposed!.getResult()
  ).resolves.toStrictEqual({ name: 'hi' })
})

test('rules', async () => {
  const model = reactive({ name: '', nickname: '', age: 12 })
  const wrapper = mount(SmartForm, {
    props: {
      formSchema: [
        useCustomItem({
          component: BasicComp,
          key: 'name',
        }),
        useCustomItem({
          component: CustomModelComp,
          key: 'custom',
          modelKeyMap: {
            model: 'nickname',
            count: 'age',
          },
        }),
      ],
      modelValue: model,
      rules: {
        name: [{ required: true }],
        nickname: { required: true },
        age: { required: true, min: 18, type: 'number' },
      },
    },
    global: {
      plugins: [createSmartForm()],
    },
  })

  const matchError = async (err: string) => {
    await expect(
      wrapper.getComponent(SmartForm).vm.$.exposed!.getResult()
    ).rejects.toThrowError(err)
  }

  await matchError('name is required')

  await wrapper.find('input').setValue('hello')

  await matchError('nickname is required')

  await wrapper.find('input.model').setValue('world')

  await matchError('age cannot be less than 18')
  await wrapper.find('input.count').setValue('18')

  expect(
    wrapper.getComponent(SmartForm).vm.$.exposed!.getResult()
  ).resolves.toStrictEqual({ name: 'hello', nickname: 'world', age: 18 })
})

test('rules in FormItem', async () => {
  const model = reactive({ name: '', nickname: '', age: 12, tag: '' })
  const wrapper = mount(SmartForm, {
    props: {
      formSchema: [
        useCustomItem({
          component: BasicComp,
          key: 'name',
          rules: [{ required: true }],
        }),
        useCustomItem({
          component: BasicComp,
          key: 'tag',
          rules: [
            {
              //callback style
              asyncValidator(r, v, cb) {
                setTimeout(() => {
                  if (!model.tag) {
                    cb('What kind of fan are you?')
                  }
                  cb()
                }, 10)
              },
            },
          ],
        }),
        useCustomItem({
          component: CustomModelComp,
          key: 'custom',
          modelKeyMap: {
            model: 'nickname',
            count: 'age',
          },
          rules: {
            // custom async rule
            asyncValidator(r, v, cb, source, options) {
              if (model.age < 18) {
                throw new Error(`It's a grown-up's world, not a kid's.`)
              }
              return Promise.resolve()
            },
          },
        }),
      ],
      modelValue: model,
    },
    global: {
      plugins: [createSmartForm()],
    },
  })

  const matchError = async (err: string, checkMsg = true) => {
    await expect(
      wrapper.getComponent(SmartForm).vm.$.exposed!.getResult()
    ).rejects.toThrowError(err)

    if (checkMsg) {
      const handler = wrapper.find('.error-msg')
      expect(handler.exists()).true
      expect(handler.text()).toBe(err)
    }
  }

  // 没操作不显示错误信息
  expect(wrapper.find('.error-msg').exists()).toBe(false)
  await matchError('name is required')

  await wrapper.find('input').setValue('Trump')

  await matchError('What kind of fan are you?')

  await wrapper.find('#tag input').setValue('MAGA')

  await matchError(`It's a grown-up's world, not a kid's.`)

  await wrapper.find('input.count').setValue('18')

  expect(
    wrapper.getComponent(SmartForm).vm.$.exposed!.getResult()
  ).resolves.toStrictEqual({
    name: 'Trump',
    nickname: '',
    tag: 'MAGA',
    age: 18,
  })
})
